package cn.iocoder.yudao.module.reward.service.redeem;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.reward.dal.dataobject.redeem.RewardRedeemDO;
import cn.iocoder.yudao.module.reward.dal.mysql.redeem.RewardRedeemMapper;
import cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.reward.enums.RewardPickMethedEnum;
import cn.iocoder.yudao.module.reward.enums.RewardRedeemStatusEnum;
import cn.iocoder.yudao.module.reward.service.reward.RewardService;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageReqVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageRespVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemUpdateVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemVerifyVO;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.micrometer.core.instrument.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;

/**
 * 礼品兑换 Service
 *
 * @author 系统管理员
 */
@Service
@Slf4j
public class RewardRedeemServiceImpl extends AbstractService<RewardRedeemMapper, RewardRedeemDO> implements RewardRedeemService {
    @Resource
    private RewardRedeemMapper rewardRedeemMapper;
    @Resource
    private RewardService rewardService;

    @Override
    public PageResult<RewardRedeemPageRespVO> pageInfo(RewardRedeemPageReqVO reqVO) {
        int start = (reqVO.getPageNo() - 1) * reqVO.getPageSize();
        int size = reqVO.getPageSize();
        List<RewardRedeemPageRespVO > result = rewardRedeemMapper.pageInfo(start, size, reqVO);
        int total = rewardRedeemMapper.pageCount(reqVO);
        return new PageResult<>(result, total, reqVO.getPageSize(), reqVO.getPageNo(), (total + reqVO.getPageSize() - 1) / reqVO.getPageSize());
    }

    @Override
    public RewardRedeemPageRespVO detail(Long id) {
        if (id == null) {
            return null;
        }
        return rewardRedeemMapper.detail(id);
    }

    @Override
    public Boolean updateRecord(RewardRedeemUpdateVO request) {
        updateValidateParam(request);
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getRedeemType, request.getRedeemType());
        wrapper.set(RewardRedeemDO::getExpenses, request.getExpenses());
        wrapper.set(RewardRedeemDO::getCurrency, request.getCurrency());
        wrapper.set(RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
        wrapper.set(RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(RewardRedeemDO::getRedeemer, request.getRedeemer());
        wrapper.set(RewardRedeemDO::getRedemptionTime, request.getRedemptionTime());
        wrapper.set(RewardRedeemDO::getCourierCompany, request.getCourierCompany());
        wrapper.set(RewardRedeemDO::getExpressNo, request.getExpressNo());
        wrapper.set(RewardRedeemDO::getExpressDate, request.getExpressDate());
        wrapper.set(RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getAnnex, request.getAnnex());
        wrapper.set(RewardRedeemDO::getRemark, request.getRemark());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    @Override
    public Boolean verify(RewardRedeemVerifyVO request) {
        verifyValidateParam(request);
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMED.getValue());
        wrapper.set(RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
        wrapper.set(RewardRedeemDO::getCourierCompany, request.getCourierCompany());
        wrapper.set(RewardRedeemDO::getExpressNo, request.getExpressNo());
        wrapper.set(RewardRedeemDO::getExpressDate, request.getExpressDate());
        wrapper.set(RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getVerifyUser, request.getVerifyUser());
        wrapper.set(RewardRedeemDO::getVerifyTime, request.getVerifyTime());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    private void verifyValidateParam(RewardRedeemVerifyVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_STATUS_ERROR);
        }
        if (rewardRedeemDO.getRedeemType() != RewardPickMethedEnum.PICKUP.getValue()) {
            //除上门领取状态外，其他方式需要验证 courierCompany, expressNo, expressDate, expressSender, recipientAddress 不能为空
            if (request.getCourierCompany() == null) {
                throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
            }
            if (StringUtils.isBlank(request.getExpressNo())) {
                throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
            }
            if (request.getExpressDate() == null) {
                throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
            }
            if (StringUtils.isBlank(request.getExpressSender())) {
                throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
            }
            if (StringUtils.isBlank(request.getRecipientAddress())) {
                throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
            }
        }
    }

    private void updateValidateParam(RewardRedeemUpdateVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_STATUS_ERROR);
        }
    }
}
